@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -52,6 +24,7 @@ package com.openexchange.admin.reseller.rmi;
 import java.rmi.Remote;
 import java.rmi.RemoteException;
 import java.util.HashSet;
+import java.util.Set;
 import com.openexchange.admin.reseller.rmi.dataobjects.ResellerAdmin;
 import com.openexchange.admin.reseller.rmi.dataobjects.Restriction;
 import com.openexchange.admin.reseller.rmi.exceptions.OXResellerException;
@@ -62,7 +35,10 @@ import com.openexchange.admin.rmi.exceptions.InvalidDataException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
 
 /**
+ * {@link OXResellerInterface}
+ * 
  * @author choeger
+ * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
  */
 public interface OXResellerInterface extends Remote {
 
@@ -73,14 +49,16 @@ public interface OXResellerInterface extends Remote {
 
     /**
      * Creates a ResellerAdmin subadmin account in the database.
-     *  Example:
-     *  final ResellerAdmin ra = new ResellerAdmin("reselleradm");
-     *  ra.setDisplayname("Reseller Admin");
-     *  ra.setPassword("secret");
-     *  HashSet<Restriction> rss = new HashSet<Restriction>();
-     *  rss.add(new Restriction(Restriction.MAX_CONTEXT_PER_SUBADMIN, "100"));
-     *  rss.add(new Restriction(Restriction.MAX_OVERALL_USER_PER_SUBADMIN,"1000"));
-     *
+     * Example:
+     * <pre>
+     * final ResellerAdmin ra = new ResellerAdmin("reselleradm");
+     * ra.setDisplayname("Reseller Admin");
+     * ra.setPassword("secret");
+     * HashSet<Restriction> rss = new HashSet<Restriction>();
+     * rss.add(new Restriction(Restriction.MAX_CONTEXT_PER_SUBADMIN, "100"));
+     * rss.add(new Restriction(Restriction.MAX_OVERALL_USER_PER_SUBADMIN,"1000"));
+     * </pre>
+     * 
      * @param adm {@link ResellerAdmin} object
      * @param creds {@link Credentials} of the master admin
      * @return {@link ResellerAdmin} object
@@ -90,7 +68,7 @@ public interface OXResellerInterface extends Remote {
      * @throws InvalidCredentialsException
      * @throws OXResellerException
      */
-    public ResellerAdmin create(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException, OXResellerException;
+    ResellerAdmin create(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException, OXResellerException;
 
     /**
      * Delete ResellerAdmin from database If any objects still belong to this subadmin, the deletion will fail
@@ -103,7 +81,7 @@ public interface OXResellerInterface extends Remote {
      * @throws OXResellerException
      * @throws InvalidCredentialsException
      */
-    public void delete(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException;
+    void delete(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException;
 
     /**
      * Change ResellerAdmin parameters
@@ -116,7 +94,20 @@ public interface OXResellerInterface extends Remote {
      * @throws OXResellerException
      * @throws InvalidCredentialsException
      */
-    public void change(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException;
+    void change(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException;
+
+    /**
+     * Allows a reseller admin to change his own capabilities, properties and taxonomies
+     *
+     * @param admin The {@link ResellerAdmin} object
+     * @param credentials {@link Credentials} of the reseller admin
+     * @throws RemoteException
+     * @throws InvalidDataException
+     * @throws StorageException
+     * @throws OXResellerException
+     * @throws InvalidCredentialsException
+     */
+    void changeSelf(ResellerAdmin admin, Credentials credentials) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException;
 
     /**
      * Search for specific or all subadmins in the database
@@ -129,7 +120,7 @@ public interface OXResellerInterface extends Remote {
      * @throws StorageException
      * @throws InvalidCredentialsException
      */
-    public ResellerAdmin[] list(final String search_pattern, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException;
+    ResellerAdmin[] list(final String search_pattern, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException;
 
     /**
      * Get complete data from all subadmin objects contained in array. It is required to
@@ -144,7 +135,7 @@ public interface OXResellerInterface extends Remote {
      * @throws StorageException
      * @throws OXResellerException
      */
-    public ResellerAdmin[] getMultipleData(final ResellerAdmin[] admins, Credentials creds) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException;
+    ResellerAdmin[] getMultipleData(final ResellerAdmin[] admins, Credentials creds) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException;
 
     /**
      * Get complete data from subadmin object. It is required to either specify subadmin objects name or id.
@@ -158,7 +149,21 @@ public interface OXResellerInterface extends Remote {
      * @throws InvalidCredentialsException
      * @throws InvalidDataException
      */
-    public ResellerAdmin getData(final ResellerAdmin adm, Credentials creds) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException;
+    ResellerAdmin getData(final ResellerAdmin adm, Credentials creds) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException;
+
+    /**
+     * Returns data for the specified reseller admin
+     *
+     * @param admin The reseller admin to return data for
+     * @param credentials The reseller admin's credentials
+     * @return The reseller admin with the data
+     * @throws RemoteException
+     * @throws InvalidDataException
+     * @throws InvalidCredentialsException
+     * @throws StorageException
+     * @throws OXResellerException
+     */
+    ResellerAdmin getSelfData(ResellerAdmin admin, Credentials credentials) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException;
 
     /**
      * Retrieve a list of all currently available {@link Restriction} objects
@@ -170,7 +175,7 @@ public interface OXResellerInterface extends Remote {
      * @throws StorageException
      * @throws OXResellerException
      */
-    public Restriction[] getAvailableRestrictions(final Credentials creds) throws RemoteException, InvalidCredentialsException, StorageException, OXResellerException;
+    Restriction[] getAvailableRestrictions(final Credentials creds) throws RemoteException, InvalidCredentialsException, StorageException, OXResellerException;
 
     /**
      * Retrieve a list of all restrictions applied to given {@link Context}
@@ -184,7 +189,7 @@ public interface OXResellerInterface extends Remote {
      * @throws StorageException
      * @throws InvalidCredentialsException
      */
-    public Restriction[] getRestrictionsFromContext(final Context ctx, final Credentials creds) throws RemoteException, InvalidDataException, OXResellerException, StorageException, InvalidCredentialsException;
+    Restriction[] getRestrictionsFromContext(final Context ctx, final Credentials creds) throws RemoteException, InvalidDataException, OXResellerException, StorageException, InvalidCredentialsException;
 
     /**
      * Initialize the database with all currently possible restrictions.
@@ -197,7 +202,7 @@ public interface OXResellerInterface extends Remote {
      * @throws InvalidCredentialsException
      * @throws OXResellerException
      */
-    public void initDatabaseRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
+    void initDatabaseRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
 
     /**
      * Remove all restrictions from database
@@ -208,7 +213,7 @@ public interface OXResellerInterface extends Remote {
      * @throws StorageException
      * @throws OXResellerException
      */
-    public void removeDatabaseRestrictions(final Credentials creds) throws RemoteException, InvalidCredentialsException, StorageException, OXResellerException;
+    void removeDatabaseRestrictions(final Credentials creds) throws RemoteException, InvalidCredentialsException, StorageException, OXResellerException;
 
     /**
      * Update all restrictions based on module access combinations in case of changes to
@@ -220,7 +225,7 @@ public interface OXResellerInterface extends Remote {
      * @throws InvalidCredentialsException
      * @throws OXResellerException
      */
-    public void updateDatabaseModuleAccessRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
+    void updateDatabaseModuleAccessRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
 
     /**
      * Update list of restrictions. This is going to add new restrictions that might ship with
@@ -232,5 +237,19 @@ public interface OXResellerInterface extends Remote {
      * @throws InvalidCredentialsException
      * @throws OXResellerException
      */
-    public void updateDatabaseRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
+    void updateDatabaseRestrictions(final Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException;
+
+    /**
+     * Retrieves all capabilities for the reseller with the specified identifier
+     * 
+     * @param admin the reseller admin
+     * @param credentials {@link Credentials} of the reseller admin
+     * @return The capabilities
+     * @throws RemoteException
+     * @throws StorageException
+     * @throws InvalidCredentialsException
+     * @throws OXResellerException
+     * @throws InvalidDataException
+     */
+    Set<String> getCapabilities(ResellerAdmin admin, Credentials credentials) throws RemoteException, StorageException, InvalidDataException, InvalidCredentialsException, OXResellerException;
 }
