@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -58,6 +30,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Optional;
+import java.util.Set;
 import com.openexchange.admin.daemons.ClientAdminThread;
 import com.openexchange.admin.plugins.PluginException;
 import com.openexchange.admin.reseller.daemons.ClientAdminThreadExtended;
@@ -73,7 +46,6 @@ import com.openexchange.admin.reseller.services.PluginInterfaces;
 import com.openexchange.admin.reseller.storage.interfaces.OXResellerStorageInterface;
 import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Credentials;
-import com.openexchange.admin.rmi.exceptions.AbstractAdminRmiException;
 import com.openexchange.admin.rmi.exceptions.EnforceableDataObjectException;
 import com.openexchange.admin.rmi.exceptions.InvalidCredentialsException;
 import com.openexchange.admin.rmi.exceptions.InvalidDataException;
@@ -86,7 +58,10 @@ import com.openexchange.admin.tools.GenericChecks;
 import com.openexchange.exception.LogLevel;
 
 /**
+ * {@link OXReseller}
+ * 
  * @author <a href="mailto:carsten.hoeger@open-xchange.com">Carsten Hoeger</a>
+ * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
  */
 public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
@@ -97,6 +72,11 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
     private final OXResellerStorageInterface oxresell;
     private final ResellerAuth resellerauth;
 
+    /**
+     * Initializes a new {@link OXReseller}.
+     * 
+     * @throws StorageException if an error is occurred
+     */
     public OXReseller() throws StorageException {
         super();
         log(LogLevel.DEBUG, LOGGER, null, null, "Class loaded: {}", this.getClass().getName());
@@ -111,31 +91,10 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
         }
     }
 
-    private void logAndEnhanceException(Throwable t, final Credentials credentials) {
-        if (t instanceof AbstractAdminRmiException) {
-            logAndReturnException(LOGGER, ((AbstractAdminRmiException) t), credentials);
-        } else if (t instanceof RemoteException) {
-            RemoteException remoteException = (RemoteException) t;
-            String exceptionId = AbstractAdminRmiException.generateExceptionId();
-            RemoteExceptionUtils.enhanceRemoteException(remoteException, exceptionId);
-            logAndReturnException(LOGGER, remoteException, exceptionId, credentials);
-        } else if (t instanceof Exception) {
-            RemoteException remoteException = RemoteExceptionUtils.convertException((Exception) t);
-            String exceptionId = AbstractAdminRmiException.generateExceptionId();
-            RemoteExceptionUtils.enhanceRemoteException(remoteException, exceptionId);
-            logAndReturnException(LOGGER, remoteException, exceptionId, credentials);
-        }
-    }
-
     @Override
     public void change(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException {
         try {
-            try {
-                doNullCheck(adm);
-            } catch (InvalidDataException e) {
-                log(LogLevel.ERROR, LOGGER, creds, e, "Invalid data sent by client!");
-                throw e;
-            }
+            doNullCheck(LOGGER, creds, adm);
             ResellerAdmin parent = null;
             final Credentials masterCredentials = cache.getMasterCredentials();
             if (null != masterCredentials && masterCredentials.getLogin().equals(creds.getLogin())) {
@@ -235,7 +194,39 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
                 }
             }
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
+            throw e;
+        }
+    }
+
+    @Override
+    public void changeSelf(ResellerAdmin admin, Credentials credentials) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException {
+        try {
+            doNullCheck(LOGGER, credentials, admin);
+            checkIdOrName(admin);
+            if (admin.getId() != null && !oxresell.existsAdmin(admin)) {
+                throw new OXResellerException(Code.RESELLER_ADMIN_NOT_EXIST, admin.getName());
+            }
+
+            ResellerAdmin dbAdmin = oxresell.getData(new ResellerAdmin[] { admin })[0];
+            Credentials masterCredentials = cache.getMasterCredentials();
+            if (null != masterCredentials && masterCredentials.getLogin().equals(credentials.getLogin())) {
+                basicauth.doAuthentication(credentials);
+            } else {
+                if (!dbAdmin.getName().equalsIgnoreCase(credentials.getLogin())) {
+                    InvalidCredentialsException invalidCredentialsException = new InvalidCredentialsException("Authentication failed");
+                    log(LogLevel.ERROR, LOGGER, credentials, invalidCredentialsException, "changeSelf can only be applied to own data");
+                    throw invalidCredentialsException;
+                }
+                resellerauth.doAuthentication(credentials);
+                if (admin.getParentId() != null) {
+                    throw new OXResellerException(Code.SUBAMIN_NOT_ALLOWED_TO_CHANGE_PARENTID);
+                }
+            }
+            checkResellerChangeSelfData(admin, dbAdmin);
+            oxresell.change(admin);
+        } catch (Throwable e) {
+            enhanceAndLogException(e, credentials);
             throw e;
         }
     }
@@ -243,12 +234,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
     @Override
     public ResellerAdmin create(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException, OXResellerException {
         try {
-            try {
-                doNullCheck(adm);
-            } catch (InvalidDataException e) {
-                log(LogLevel.ERROR, LOGGER, creds, e, "Invalid data sent by client!");
-                throw e;
-            }
+            doNullCheck(LOGGER, creds, adm);
             ResellerAdmin parent = null;
 
             final Credentials masterCredentials = cache.getMasterCredentials();
@@ -290,7 +276,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             // Trigger plugin extensions
             {
-                final List<OXResellerPluginInterface> interfacelist = new ArrayList<OXResellerPluginInterface>();
+                final List<OXResellerPluginInterface> interfacelist = new ArrayList<>();
                 final PluginInterfaces pluginInterfaces = PluginInterfaces.getInstance();
                 if (null != pluginInterfaces) {
                     for (final OXResellerPluginInterface oxresellpi : pluginInterfaces.getResellerPlugins().getServiceList()) {
@@ -326,7 +312,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
             logAndReturnException(LOGGER, remoteException, e.getExceptionId(), creds);
             throw remoteException;
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -334,12 +320,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
     @Override
     public void delete(final ResellerAdmin adm, final Credentials creds) throws RemoteException, InvalidDataException, StorageException, OXResellerException, InvalidCredentialsException {
         try {
-            try {
-                doNullCheck(adm);
-            } catch (InvalidDataException e) {
-                log(LogLevel.ERROR, LOGGER, creds, e, "Invalid data sent by client!");
-                throw e;
-            }
+            doNullCheck(LOGGER, creds, adm);
             boolean isMaster = false;
             final Credentials masterCredentials = cache.getMasterCredentials();
             if (null != masterCredentials && masterCredentials.getLogin().equals(creds.getLogin())) {
@@ -374,9 +355,14 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
                 throw new OXResellerException(Code.UNABLE_TO_DELETE, dbadm.getId().toString());
             }
 
+            ResellerAdmin[] list = oxresell.list("*", i(dbadm.getId()));
+            if (list.length != 0) {
+                throw new OXResellerException(Code.UNABLE_TO_DELETE_OWNS_SUBADMINS, dbadm.getId().toString());
+            }
+
             dbadm.setParentName(parent.isPresent() ? parent.get().getName() : null);
 
-            final ArrayList<OXResellerPluginInterface> interfacelist = new ArrayList<OXResellerPluginInterface>();
+            final ArrayList<OXResellerPluginInterface> interfacelist = new ArrayList<>();
 
             // Trigger plugin extensions
             {
@@ -398,7 +384,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             oxresell.delete(dbadm);
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -421,13 +407,13 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
                 throw new OXResellerException(Code.UNABLE_TO_LOAD_AVAILABLE_RESTRICTIONS_FROM_DATABASE);
             }
 
-            final HashSet<Restriction> ret = new HashSet<Restriction>();
+            final HashSet<Restriction> ret = new HashSet<>();
             for (final Entry<String, Restriction> entry : validRestrictions.entrySet()) {
                 ret.add(entry.getValue());
             }
             return ret.toArray(new Restriction[ret.size()]);
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -437,18 +423,35 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
         return getMultipleData(new ResellerAdmin[] { adm }, creds)[0];
     }
 
+    @Override
+    public ResellerAdmin getSelfData(ResellerAdmin admin, Credentials credentials) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException {
+        try {
+            doNullCheck(LOGGER, credentials, admin);
+            
+            Credentials masterCredentials = cache.getMasterCredentials();
+            if (null != masterCredentials && masterCredentials.getLogin().equals(credentials.getLogin())) {
+                basicauth.doAuthentication(credentials);
+            } else {
+                resellerauth.doAuthentication(credentials);
+                ResellerAdmin parent = oxresell.getData(new ResellerAdmin[] { new ResellerAdmin(credentials.getLogin(), credentials.getPassword()) })[0];
+                ResellerAdmin dbAdmin = oxresell.getData(new ResellerAdmin[] { admin })[0];
+                if (!dbAdmin.equals(parent) && null != parent && false == dbAdmin.getParentId().equals(parent.getId())) {
+                    log(LogLevel.ERROR, LOGGER, credentials, null, "unathorized access to {} by {}", dbAdmin.getName(), credentials.getLogin());
+                    throw new InvalidCredentialsException("authentication failed");
+                }
+            }
+            checkIdOrName(admin);
+            return oxresell.getData(new ResellerAdmin[] { new ResellerAdmin(admin.getId(), admin.getName()) })[0];
+        } catch (Throwable e) {
+            enhanceAndLogException(e, credentials);
+            throw e;
+        }
+    }
+
     @Override
     public ResellerAdmin[] getMultipleData(final ResellerAdmin[] admins, final Credentials creds) throws RemoteException, InvalidDataException, InvalidCredentialsException, StorageException, OXResellerException {
         try {
-            try {
-                doNullCheck((Object[]) admins);
-                if (admins.length <= 0) {
-                    throw new InvalidDataException();
-                }
-            } catch (InvalidDataException e) {
-                log(LogLevel.ERROR, LOGGER, creds, e, "Invalid data sent by client!");
-                throw e;
-            }
+            doNullCheck(LOGGER, creds, (Object[]) admins);
             int pid = 0;
             final Credentials masterCredentials = cache.getMasterCredentials();
             if (null != masterCredentials && masterCredentials.getLogin().equals(creds.getLogin())) {
@@ -461,13 +464,21 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
             checkAdminIdOrName(admins);
             for (final ResellerAdmin admin : admins) {
                 if (!oxresell.existsAdmin(admin, pid)) {
-                    throw new OXResellerException(Code.RESELLER_ADMIN_NOT_EXIST, admin.getName());
+                    String id;
+                    if (admin.isNameset()) {
+                        id = admin.getName();
+                    } else if (admin.isIdset()) {
+                        id = admin.getId().toString();
+                    } else {
+                        id = "";
+                    }
+                    throw new OXResellerException(Code.RESELLER_ADMIN_NOT_EXIST, id);
                 }
             }
 
             return oxresell.getData(admins);
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -495,7 +506,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             return oxresell.getRestrictionsFromContext(ctx);
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -510,7 +521,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
             }
             oxresell.initDatabaseRestrictions();
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -533,7 +544,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
             int pid = oxresell.getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0].getId().intValue();
             return oxresell.list(search_pattern, pid);
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -549,7 +560,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             oxresell.removeDatabaseRestrictions();
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -566,7 +577,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             oxresell.updateModuleAccessRestrictions();
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
@@ -598,7 +609,7 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
      */
     private void checkIdOrName(final ResellerAdmin adm) throws InvalidDataException {
         if (adm.getId() == null && adm.getName() == null) {
-            throw new InvalidDataException("either ID or name must be specified");
+            throw new InvalidDataException("Either ID or name must be specified");
         }
     }
 
@@ -630,6 +641,25 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
         }
     }
 
+    /**
+     * Checks if any contained reseller data besides ID, Capabilities, Properties, Taxonomies is set
+     * for a self-change.
+     *
+     * @param admin The admin that issued the self-change
+     * @throws InvalidDataException If invalid data is set
+     */
+    private void checkResellerChangeSelfData(ResellerAdmin admin, ResellerAdmin dbAdmin) throws InvalidDataException {
+        // @formatter:off
+        if (admin.isNameset() && false == admin.getName().equals(dbAdmin.getName())) {
+            throw new InvalidDataException("Invalid data sent by client!");
+        }
+        if ( admin.isDisplaynameset() || admin.isParentIdset() || admin.isParentNameset() || admin.isPasswordMechset() 
+            || admin.isPasswordset() || admin.isRestrictionsset() || admin.isSaltSet()) {
+            throw new InvalidDataException("Invalid data sent by client!");
+        }
+        // @formatter:on
+    }
+
     @Override
     public void updateDatabaseRestrictions(Credentials creds) throws RemoteException, StorageException, InvalidCredentialsException, OXResellerException {
         try {
@@ -642,9 +672,70 @@ public class OXReseller extends OXCommonImpl implements OXResellerInterface {
 
             oxresell.updateRestrictions();
         } catch (Throwable e) {
-            logAndEnhanceException(e, creds);
+            enhanceAndLogException(e, creds);
             throw e;
         }
     }
 
+    @Override
+    public Set<String> getCapabilities(ResellerAdmin admin, Credentials credentials) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException, OXResellerException {
+        try {
+            doNullCheck(LOGGER, credentials, admin);
+
+            ResellerAdmin parent = null;
+            Credentials masterCredentials = cache.getMasterCredentials();
+            boolean isMaster = false;
+            if (null != masterCredentials && masterCredentials.getLogin().equals(credentials.getLogin())) {
+                basicauth.doAuthentication(credentials);
+                isMaster = true;
+            } else {
+                resellerauth.doAuthentication(credentials);
+                parent = oxresell.getData(new ResellerAdmin[] { new ResellerAdmin(credentials.getLogin(), credentials.getPassword()) })[0];
+                if (admin.getParentId() != null) {
+                    throw new OXResellerException(Code.SUBADMIN_IS_NOT_AUTHORIZED_TO_LIST_DATA);
+                }
+            }
+
+            checkIdOrName(admin);
+            if (admin.getId() != null && !oxresell.existsAdmin(admin)) {
+                throw new OXResellerException(Code.RESELLER_ADMIN_NOT_EXIST, admin.getName());
+            }
+
+            if (isMaster || null == parent || parent.getId().equals(admin.getId())) {
+                return getCapabilities(admin);
+            }
+
+            GenericChecks.checkChangeValidPasswordMech(admin);
+            ResellerAdmin dbAdmin = oxresell.getData(new ResellerAdmin[] { admin })[0];
+            if (false == dbAdmin.getParentId().equals(parent.getId())) {
+                LOGGER.error("Unathorized access to {} by {}", dbAdmin.getName(), credentials.getLogin());
+                throw new InvalidCredentialsException("Authentication failed");
+            }
+            // if no password mech supplied, use the old one as set in db
+            if (admin.getPasswordMech() == null) {
+                admin.setPasswordMech(dbAdmin.getPasswordMech());
+            }
+
+            return getCapabilities(admin);
+        } catch (Throwable e) {
+            enhanceAndLogException(e, credentials);
+            throw e;
+        }
+    }
+
+    /**
+     * Returns the capabilities for the specified admin
+     *
+     * @param admin
+     * @return The capabilities
+     * @throws StorageException
+     */
+    private Set<String> getCapabilities(ResellerAdmin admin) throws StorageException {
+        ResellerAdmin[] data = oxresell.getData(new ResellerAdmin[] { admin });
+        Set<String> c = new HashSet<>();
+        for (ResellerAdmin d : data) {
+            c.addAll(d.getCapabilities());
+        }
+        return c;
+    }
 }
