@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -103,7 +75,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void change(final Context ctx, final Credentials auth) throws PluginException {
-        if (!cache.isMasterAdmin(auth)) {
+        if (!cache.isMasterAdmin(auth, false)) {
             checkOwnerShipAndSetSid(ctx, auth);
         }
         final OXContextExtensionImpl firstExtensionByName = (OXContextExtensionImpl) ctx.getFirstExtensionByName(OXContextExtensionImpl.class.getName());
@@ -130,7 +102,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void changeModuleAccess(final Context ctx, final UserModuleAccess access, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
 
@@ -145,7 +117,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void changeModuleAccess(final Context ctx, final String access_combination_name, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
 
@@ -160,7 +132,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void changeCapabilities(Context ctx, Set<String> capsToAdd, Set<String> capsToRemove, Set<String> capsToDrop, Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
 
@@ -175,7 +147,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void changeQuota(Context ctx, String module, long quotaValue, Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
 
@@ -196,7 +168,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
             throw new PluginException(e1);
         }
         applyRestrictionsPerContext(ctx);
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             try {
                 oxresell.writeCustomId(ctx);
             } catch (StorageException e) {
@@ -254,7 +226,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public Map<String, Object> undoableDelete(Context ctx, Credentials auth) throws PluginException {
-        boolean ismasteradmin = cache.isMasterAdmin(auth);
+        boolean ismasteradmin = cache.isMasterAdmin(auth, false);
         try {
             Map<String, Object> undoInfo = new HashMap<String, Object>(4);
             if (ismasteradmin) {
@@ -301,7 +273,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void disable(final Context ctx, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         checkOwnerShipAndSetSid(ctx, auth);
@@ -309,7 +281,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void disableAll(final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
 
@@ -325,7 +297,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void downgrade(final Context ctx, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         checkOwnerShipAndSetSid(ctx, auth);
@@ -333,7 +305,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void enable(final Context ctx, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         checkOwnerShipAndSetSid(ctx, auth);
@@ -341,7 +313,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void enableAll(final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         try {
@@ -355,7 +327,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public String getAccessCombinationName(final Context ctx, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return null;
         }
         checkOwnerShipAndSetSid(ctx, auth);
@@ -364,9 +336,10 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public List<OXCommonExtension> getData(final List<Context> ctxs, final Credentials auth) throws PluginException {
-        final ArrayList<OXCommonExtension> retval = new ArrayList<OXCommonExtension>();
+        boolean masterAdmin = cache.isMasterAdmin(auth, false);
+        List<OXCommonExtension> retval = new ArrayList<OXCommonExtension>(ctxs.size());
         for (final Context ctx : ctxs) {
-            if (cache.isMasterAdmin(auth)) {
+            if (masterAdmin) {
                 try {
                     final OXContextExtensionImpl ctxext = new OXContextExtensionImpl(oxresell.getContextOwner(ctx), oxresell.getRestrictionsFromContext(ctx));
                     ctxext.setCustomid(oxresell.getCustomId(ctx));
@@ -394,7 +367,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public UserModuleAccess getModuleAccess(final Context ctx, final Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return null;
         }
         checkOwnerShipAndSetSid(ctx, auth);
@@ -414,12 +387,13 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
     @Override
     public Filter<Integer, Integer> filter(final Credentials auth) throws PluginException {
         try {
-            if ( ! ClientAdminThreadExtended.cache.isMasterAdmin(auth) ) {
-                ResellerAdmin adm = null;
-                adm = (this.oxresell.getData(new ResellerAdmin[]{ new ResellerAdmin(auth.getLogin()) } ))[0];
-                return new ResellerContextFilter(cache, adm);
+            if (ClientAdminThreadExtended.cache.isMasterAdmin(auth, false) ) {
+                return null;
             }
-            return null;
+
+            ResellerAdmin adm = null;
+            adm = (this.oxresell.getData(new ResellerAdmin[]{ new ResellerAdmin(auth.getLogin()) } ))[0];
+            return new ResellerContextFilter(cache, adm);
         } catch (StorageException e) {
             throw new PluginException(e);
         }
@@ -490,7 +464,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void exists(Context ctx, Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         if (null == ctx.getId()) {
@@ -507,7 +481,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void existsInServer(Context ctx, Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         if (null == ctx.getId()) {
@@ -524,7 +498,7 @@ public class OXResellerContextImpl implements OXContextPluginInterfaceExtended {
 
     @Override
     public void getAdminId(Context ctx, Credentials auth) throws PluginException {
-        if (cache.isMasterAdmin(auth)) {
+        if (cache.isMasterAdmin(auth, false)) {
             return;
         }
         checkOwnerShipAndSetSid(ctx, auth);
